/**
 * OpenGL 3.x with GLEW and GLUT
 *
 * author	Po-Ying Li bylee@nchc.org.tw
 * version	1.0
 *
 * Homepage: http://viml.nchc.org.tw
 *
 * Copyright Po-Ying Li
 */
#ifndef _MYTRANSFORM
#define _MYTRANSFORM

void Scale(Matrix& mResult,const Vector& a)
{
	mResult.m44[0][0] = a.v[0];
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = 0;
	mResult.m44[3][0] = 0;

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = a.v[1];
	mResult.m44[2][1] = 0;
	mResult.m44[3][1] = 0;

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = a.v[2];
	mResult.m44[3][2] = 0;

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = 0;
	mResult.m44[3][3] = 1;
}

void Translate(Matrix& mResult,const Vector& a)
{
	mResult.m44[0][0] = 1;
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = 0;
	mResult.m44[3][0] = a.v[0];

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = 1;
	mResult.m44[2][1] = 0;
	mResult.m44[3][1] = a.v[1];

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = 1;
	mResult.m44[3][2] = a.v[2];

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = 0;
	mResult.m44[3][3] = 1;
}

void Rotate(Matrix& mResult,float angle,const Vector& a)
{
	float c,s;
	float fLength;
	float fRatio;
	Vector vNormalize;
	c = cos(angle*PI/180.0f);
	s = sin(angle*PI/180.0f);
	fLength = a.v[0]*a.v[0] + a.v[1]*a.v[1] + a.v[2]*a.v[2];
	fLength = sqrt(fLength);
	fRatio = 1.0/fLength;

	vNormalize.v[0] = a.v[0] * fRatio;
	vNormalize.v[1] = a.v[1] * fRatio;
	vNormalize.v[2] = a.v[2] * fRatio;

	mResult.m44[0][0] = vNormalize.v[0]*vNormalize.v[0]*(1-c) + c;
	mResult.m44[1][0] = vNormalize.v[0]*vNormalize.v[1]*(1-c) - vNormalize.v[2]*s;
	mResult.m44[2][0] = vNormalize.v[0]*vNormalize.v[2]*(1-c) + vNormalize.v[1]*s;
	mResult.m44[3][0] = 0;

	mResult.m44[0][1] = vNormalize.v[1]*vNormalize.v[0]*(1-c) + vNormalize.v[2]*s;
	mResult.m44[1][1] = vNormalize.v[1]*vNormalize.v[1]*(1-c) + c;
	mResult.m44[2][1] = vNormalize.v[1]*vNormalize.v[2]*(1-c) - vNormalize.v[0]*s;
	mResult.m44[3][1] = 0;

	mResult.m44[0][2] = vNormalize.v[0]*vNormalize.v[2]*(1-c) - vNormalize.v[1]*s;
	mResult.m44[1][2] = vNormalize.v[1]*vNormalize.v[2]*(1-c) + vNormalize.v[0]*s;
	mResult.m44[2][2] = vNormalize.v[2]*vNormalize.v[2]*(1-c) + c;
	mResult.m44[3][2] = 0;

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = 0;
	mResult.m44[3][3] = 1;
}

#endif